from bears.css.CSSAutoPrefixBear import CSSAutoPrefixBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
.example {
    display: -webkit-box;
    display: -webkit-flex;
    display: -ms-flexbox;
    display: flex;
}
""".splitlines(keepends=True)

bad_file = """
.example {
    display: flex;
}
""".splitlines(keepends=True)

CSSAutoPrefixBear = verify_local_bear(CSSAutoPrefixBear,
                                      valid_files=(good_file,),
                                      invalid_files=(bad_file,))



from bears.vhdl.VHDLLintBear import VHDLLintBear
from tests.LocalBearTestHelper import verify_local_bear


VHDLLintBearTest = verify_local_bear(VHDLLintBear,
                                     (('test',),),
                                     (('\t',),))



import os
import unittest

from queue import Queue


from bears.general.CPDBear import CPDBear
from tests.BearTestHelper import generate_skip_decorator
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


@generate_skip_decorator(CPDBear)
class CPDBearTest(unittest.TestCase):

    def setUp(self):
        self.base_test_path = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            "code_duplication_samples"))

        self.section = Section("default")
        self.section.append(Setting("language", "java"))
        self.queue = Queue()

    def test_good_file(self):
        good_file = os.path.join(self.base_test_path, "good_code.java")

        with open(good_file) as file:
            good_filelines = file.readlines()

        self.uut = CPDBear({good_file: good_filelines},
                           self.section,
                           self.queue)

        result = list(self.uut.run_bear_from_section([], {}))

        self.assertEqual(result, [])

    def test_bad_file(self):

        bad_file = os.path.join(self.base_test_path, "bad_code.java")

        with open(bad_file) as file:
            bad_filelines = file.readlines()

        self.uut = CPDBear({bad_file: bad_filelines},
                           self.section,
                           self.queue)

        result = list(self.uut.run_bear_from_section([], {}))

        self.assertNotEqual(result, [])



from bears.python.PyDocStyleBear import PyDocStyleBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = '''
"""example module level docstring."""


def hello():
    """Print hello world."""
    print("hello world")

'''.splitlines(keepends=True)


bad_file = '''
"""
example module level docstring
"""

def hello():
    print("hello world")

'''.splitlines(keepends=True)

PyDocStyleBearTest = verify_local_bear(
    PyDocStyleBear,
    valid_files=(good_file,),
    invalid_files=(bad_file,),
    tempfile_kwargs={"suffix": ".py"})

PyDocStyleBearIgnoreAllTest = verify_local_bear(
    PyDocStyleBear,
    valid_files=(good_file, bad_file,),
    invalid_files=(),
    settings={'pydocstyle_ignore': 'D400, D200, D103'},
    tempfile_kwargs={"suffix": ".py"})

# Checks if an invalid file yields results when only some errors are ignored.
PyDocStyleBearIgnoreSomeTest = verify_local_bear(
    PyDocStyleBear,
    valid_files=(good_file,),
    invalid_files=(bad_file,),
    settings={'pydocstyle_ignore': 'D400, D200'},
    tempfile_kwargs={"suffix": ".py"})

# Checks if an invalid file yields results when a particular error is selected.
PyDocStyleBearSelectSomeTest = verify_local_bear(
    PyDocStyleBear,
    valid_files=(good_file,),
    invalid_files=(bad_file,),
    settings={'pydocstyle_select': 'D200'},
    tempfile_kwargs={"suffix": ".py"})

# Checks if an invalid file yields nothing when the selected error never occurs.
PyDocStyleBearSelectAbsentErrorTest = verify_local_bear(
    PyDocStyleBear,
    valid_files=(good_file, bad_file,),
    invalid_files=(),
    settings={'pydocstyle_select': 'D500'},
    tempfile_kwargs={"suffix": ".py"})

PyDocStyleBearSelectAndIgnoreTest = verify_local_bear(
    PyDocStyleBear,
    valid_files=(good_file, bad_file,),
    invalid_files=(),
    settings={'pydocstyle_select': 'D200',
              'pydocstyle_ignore': 'D400'},
    tempfile_kwargs={"suffix": ".py"})



from coalib.bearlib.abstractions.Linter import linter


@linter(executable='sqlint', use_stdin=True, output_format="regex",
        output_regex=r'.+:(?P<line>\d+):(?P<column>\d+):'
                     r'(?P<severity>ERROR|WARNING) (?P<message>(?:\s*.+)*)')
class SQLintBear:
    """
    Check the given SQL files for syntax errors or warnings.

    This bear supports ANSI syntax. Check out
    <https://github.com/purcell/sqlint> for more detailed information.
    """

    LANGUAGES = "SQL"

    @staticmethod
    def create_arguments(filename, file, config_file):
        return ()



from coalib.bearlib.abstractions.Linter import linter
from coalib.settings.Setting import path


@linter(executable='cmakelint',
        output_format='regex',
        output_regex=r'.+:(?P<line>\d+): (?P<message>.*)')
class CMakeLintBear:
    """
    Checks the code with ``cmakelint``.
    """
    LANGUAGES = 'CMake'

    @staticmethod
    def create_arguments(filename, file, config_file,
                         cmakelint_config: path=""):
        """
        :param cmakelint_config: The location of the cmakelintrc config file.
        """
        args = ()
        if cmakelint_config:
            args += ('--config=' + cmakelint_config,)
        return args + (filename,)



from coalib.bearlib.abstractions.Linter import linter


@linter(executable='mcs',
        use_stdout=False,
        use_stderr=True,
        output_format='regex',
        output_regex=r'.+\((?P<line>\d+),(?P<column>\d+)\): '
                     r'(?P<severity>error|warning) \w+: (?P<message>.+)')
class CSharpLintBear:
    """
    Checks C# code for syntactical correctness using the ``mcs`` compiler.
    """

    LANGUAGES = "C#"

    @staticmethod
    def create_arguments(filename, file, config_file):
        return filename,



from coalib.bearlib.abstractions.Linter import linter


@linter(executable='vint', output_format="regex",
        output_regex=r'.+:(?P<line>\d+):(?P<column>\d+): (?P<message>.+)')
class VintBear:
    """
    Check vimscript code for possible style problems.

    See <https://github.com/Kuniwak/vint> for more information.
    """

    LANGUAGES = "VimScript"

    @staticmethod
    def create_arguments(filename, file, config_file):
        return filename,



import json

from coalib.bearlib.abstractions.Linter import linter
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


def bool_or_str(value):
    try:
        return bool(value)
    except:
        return str(value)


def bool_or_int(value):
    try:
        return bool(value)
    except:
        return int(value)


@linter(executable='jshint',
        output_format='regex',
        output_regex=r'.+?: line (?P<line>\d+), col (?P<column>\d+), '
                     r'(?P<message>.+) \((?P<severity>[EWI])\d+\)',
        severity_map={'E': RESULT_SEVERITY.MAJOR,
                      'W': RESULT_SEVERITY.NORMAL,
                      'I': RESULT_SEVERITY.INFO})
class JSHintBear:
    """
    Detect errors and potential problems in JavaScript code and to enforce
    appropriate coding conventions. For example, problems like syntax errors,
    bugs due to implicit type conversion, leaking variables and much more
    can be detected.

    For more information on the analysis visit <http://jshint.com/>
    """

    LANGUAGES = "JavaScript"

    @staticmethod
    def generate_config(filename, file,
                        prohibit_bitwise: bool=True,
                        prohibit_prototype_overwrite: bool=True,
                        force_braces: bool=True,
                        prohibit_type_coercion: bool=True,
                        future_hostile: bool=False,
                        prohibit_typeof: bool=False,
                        force_filter_forin: bool=True,
                        allow_funcscope: bool=False,
                        iterator: bool=False,
                        prohibit_arg: bool=True,
                        prohibit_comma: bool=False,
                        prohibit_non_breaking_whitespace: bool=True,
                        prohibit_new: bool=False,
                        prohibit_undefined: bool=True,
                        prohibit_groups: bool=False,
                        prohibit_variable_statements: bool=False,
                        allow_missing_semicol: bool=False,
                        allow_debugger: bool=False,
                        allow_assignment_comparisions: bool=False,
                        allow_eval: bool=False,
                        allow_global_strict: bool=False,
                        allow_increment: bool=False,
                        allow_proto: bool=False,
                        allow_scripturls: bool=False,
                        allow_singleton: bool=False,
                        allow_this_stmt: bool=False,
                        allow_with_stmt: bool=False,
                        using_mozilla: bool=False,
                        allow_noyield: bool=False,
                        allow_eqnull: bool=False,
                        allow_last_semicolon: bool=False,
                        allow_func_in_loop: bool=False,
                        allow_expr_in_assignments: bool=False,
                        use_es6_syntax: bool=False,
                        use_es3_array: bool=False,
                        environment_mootools: bool=False,
                        environment_couch: bool=False,
                        environment_jasmine: bool=False,
                        environment_jquery: bool=False,
                        environment_node: bool=False,
                        environment_qunit: bool=False,
                        environment_rhino: bool=False,
                        environment_shelljs: bool=False,
                        environment_prototypejs: bool=False,
                        environment_yui: bool=False,
                        environment_mocha: bool=True,
                        environment_module: bool=False,
                        environment_wsh: bool=False,
                        environment_worker: bool=False,
                        environment_nonstandard: bool=False,
                        environment_browser: bool=True,
                        environment_browserify: bool=False,
                        environment_devel: bool=True,
                        environment_dojo: bool=False,
                        environment_typed: bool=False,
                        environment_phantom: bool=False,
                        maxerr: int=50,
                        maxstatements: bool_or_int=False,
                        maxdepth: bool_or_int=False,
                        maxparams: bool_or_int=False,
                        maxcomplexity: bool_or_int=False,
                        shadow: bool_or_str=False,
                        prohibit_unused: bool_or_str=True,
                        allow_latedef: bool_or_str=False,
                        es_version: int=5,
                        jshint_config: str=""):
        """
        :param prohibit_bitwise:
            This option prohibits the use of bitwise operators.
        :param prohibit_prototype_overwrite:
            This options prohibits overwriting prototypes of native objects
            such as ``Array``.
        :param force_braces:
            This option requires you to always put curly braces around blocks
            in loops and conditionals.
        :param prohibit_type_coercion:
            This options prohibits the use of ``==`` and ``!=`` in favor of
            ``===`` and ``!==``.
        :param future_hostile:
            This option enables warnings about the use of identifiers which are
            defined in future versions of JavaScript.
        :param prohibit_typeof:
            This option suppresses warnings about invalid ``typeof`` operator
            values.
        :param force_filter_forin:
            This option requires all ``for in`` loops to filter object's items.
        :param iterator:
            This option suppresses warnings about the ``__iterator__``
            property.
        :param allow_funcscope:
            This option suppresses warnings about declaring variables inside of
            control structures while accessing them later from outside.
        :param prohibit_arg:
            This option prohibits the use of ``arguments.caller`` and
            ``arguments.callee``.
        :param prohibit_comma:
            This option prohibits the use of the comma operator.
        :param prohibit_non_breaking_whitespace:
            This option warns about "non-breaking whitespace characters".
        :param prohibit_new:
            This option prohibits the use of constructor functions for
            side-effects.
        :param prohibit_undefined:
            This option prohibits the use of explicitly undeclared variables.
        :param prohibit_groups:
            This option prohibits the use of the grouping operator when it is
            not strictly required.
        :param prohibit_variable_statements:
            This option forbids the use of VariableStatements.
        :param allow_missing_semicol:
            This option suppresses warnings about missing semicolons.
        :param allow_debugger:
            This option suppresses warnings about the ``debugger`` statements.
        :param allow_assignment_comparisions:
            This option suppresses warnings about the use of assignments in
            cases where comparisons are expected.
        :param allow_eval:
            This options suppresses warnings about the use of ``eval``
            function.
        :param allow_global_strict:
            This option suppresses warnings about the use of global strict
            mode.
        :param allow_increment:
            This option suppresses warnings about the use of unary increment
            and decrement operators.
        :param allow_proto:
            This option suppresses warnings about the ``__proto__`` property.
        :param allow_scripturls:
            This option suppresses warnings about the use of script-targeted
            URLs.
        :param allow_singleton:
            This option suppresses warnings about constructions like
            ``new function () { ... }`` and ``new Object;`` sometimes used to
            produce singletons.
        :param allow_this_stmt:
            This option suppresses warnings about possible strict violations
            when the code is running in strict mode and ``this`` is used in a
            non-constructor function.
        :param allow_with_stmt:
            This option suppresses warnings about the use of the ``with``
            statement.
        :param using_mozilla:
            This options tells JSHint that your code uses Mozilla JavaScript
            extensions.
        :param allow_noyield:
            This option suppresses warnings about generator functions with no
            ``yield`` statement in them.
        :param allow_eqnull:
            This option suppresses warnings about ``== null`` comparisons.
        :param allow_last_semicolon:
            This option suppresses warnings about missing semicolons for the
            last statement.
        :param allow_func_in_loop:
            This option suppresses warnings about functions inside of loops.
        :param allow_expr_in_assignments:
            This option suppresses warnings about the use of expressions where
            normally assignments or function calls are expected.
        :param use_es3_array:
            This option tells JSHintBear ES3 array elision elements, or empty
            elements are used.
        :param use_es3_array:
            This option tells JSHint ECMAScript 6 specific syntax is used.
        :param environment_mootools:
            This option defines globals exposed by the Mootools.
        :param environment_couch:
            This option defines globals exposed by CouchDB.
        :param environment_jasmine:
            This option defines globals exposed by Jasmine.
        :param environment_jquery:
            This option defines globals exposed by Jquery.
        :param environment_node:
            This option defines globals exposed by Node.
        :param environment_qunit:
            This option defines globals exposed by Qunit.
        :param environment_rhino:
            This option defines globals exposed when the code is running inside
            rhino runtime environment.
        :param environment_shelljs:
            This option defines globals exposed by the ShellJS.
        :param environment_prototypejs:
            This option defines globals exposed by the Prototype.
        :param environment_yui:
            This option defines globals exposed by the YUI JavaScript
            Framework.
        :param environment_mocha:
            This option defines globals exposed by the "BDD" and "TDD" UIs of
            the Mocha unit testing framework.
        :param environment_module:
            This option informs JSHintBear that the input code describes an
            ECMAScript 6 module.
        :param environment_wsh:
            This option defines globals available when the code is running as a
            script for the Windows Script Host.
        :param environment_worker:
            This option defines globals available when the code is running
            inside of a Web Worker.
        :param environment_nonstandard:
            This option defines non- standard but widely adopted globals such
            as ``escape`` and ``unescape``.
        :param environment_browser:
            This option defines globals exposed by modern browsers.
        :param environment_browserify:
            This option defines globals available when using the Browserify.
        :param environment_devel:
            This option defines globals that are usually used for debugging:
            ``console``, ``alert``, etc.
        :param environment_dojo:
            This option defines globals exposed by the Dojo Toolkit.
        :param environment_typed:
            This option defines globals for typed array constructors.
        :param environment_phantom:
            This option defines globals available when your core is running
            inside of the PhantomJS runtime environment.
        :param maxerr:
            This options allows you to set the maximum amount of warnings
            JSHintBear will produce before giving up. Default is 50.
        :param maxstatements:
            Maximum number of statements allowed per function.
        :param maxdepth:
            This option lets you control how nested do you want your blocks to
            be.
        :param maxparams:
            Maximum number of formal parameters allowed per function.
        :param maxcomplexity:
            Maximum cyclomatic complexity in the code.
        :param shadow:
            This option suppresses warnings about variable shadowing i.e.
            declaring a variable that had been already declared somewhere in
            the outer scope.

            - "inner" - check for variables defined in the same scope only
            - "outer" - check for variables defined in outer scopes as well
            - False - same as inner
            - True  - allow variable shadowing
        :param prohibit_unused:
            This option generates warnings when variables are defined but never
            used. This can be set to ""vars"" to only check for variables, not
            function parameters, or ""strict"" to check all variables and
            parameters.
        :param allow_latedef:
            This option prohibits the use of a variable before it was defined.
            Setting this option to "nofunc" will allow function declarations to
            be ignored.
        :param es_version:
            This option is used to specify the ECMAScript version to which the
            code must adhere to.
        """
        if not jshint_config:
            options = {"bitwise": prohibit_bitwise,
                       "freeze": prohibit_prototype_overwrite,
                       "curly": force_braces,
                       "eqeqeq": prohibit_type_coercion,
                       "futurehostile": future_hostile,
                       "notypeof": prohibit_typeof,
                       "forin": force_filter_forin,
                       "funcscope": allow_funcscope,
                       "iterator": iterator,
                       "noarg": prohibit_arg,
                       "nocomma": prohibit_comma,
                       "nonbsp": prohibit_non_breaking_whitespace,
                       "nonew": prohibit_new,
                       "undef": prohibit_undefined,
                       "singleGroups": prohibit_groups,
                       "varstmt": prohibit_variable_statements,
                       "asi": allow_missing_semicol,
                       "debug": allow_debugger,
                       "boss": allow_assignment_comparisions,
                       "evil": allow_eval,
                       "globalstrict": allow_global_strict,
                       "plusplus": allow_increment,
                       "proto": allow_proto,
                       "scripturl": allow_scripturls,
                       "supernew": allow_singleton,
                       "validthis": allow_this_stmt,
                       "withstmt": allow_with_stmt,
                       "moz": using_mozilla,
                       "noyield": allow_noyield,
                       "eqnull": allow_eqnull,
                       "lastsemic": allow_last_semicolon,
                       "loopfunc": allow_func_in_loop,
                       "expr": allow_expr_in_assignments,
                       "esnext": use_es6_syntax,
                       "elision": use_es3_array,
                       "mootools": environment_mootools,
                       "couch": environment_couch,
                       "jasmine": environment_jasmine,
                       "jquery": environment_jquery,
                       "node": environment_node,
                       "qunit": environment_qunit,
                       "rhino": environment_rhino,
                       "shelljs": environment_shelljs,
                       "prototypejs": environment_prototypejs,
                       "yui": environment_yui,
                       "mocha": environment_mocha,
                       "module": environment_module,
                       "wsh": environment_wsh,
                       "worker": environment_worker,
                       "nonstandard": environment_nonstandard,
                       "browser": environment_browser,
                       "browserify": environment_browserify,
                       "devel": environment_devel,
                       "dojo": environment_dojo,
                       "typed": environment_typed,
                       "phantom": environment_phantom,
                       "maxerr": maxerr,
                       "maxcomplexity": maxcomplexity,
                       "maxdepth": maxdepth,
                       "maxparams": maxparams,
                       "maxstatements": maxstatements,
                       "shadow": shadow,
                       "unused": prohibit_unused,
                       "latedef": allow_latedef,
                       "esversion": es_version}

            return json.dumps(options)
        else:
            return None

    @staticmethod
    def create_arguments(filename, file, config_file, jshint_config: str=""):
        """
        :param jshint_config:
            The location of the jshintrc config file. If this option is present
            all the above options are not used. Instead the .jshintrc file is
            used as the configuration file.
        """
        args = ('--verbose', filename, '--config')
        if jshint_config:
            args += (jshint_config,)
        else:
            args += (config_file,)
        return args



from bears.python.PyImportSortBear import PyImportSortBear
from tests.LocalBearTestHelper import verify_local_bear

PyImportSortBearTest = verify_local_bear(PyImportSortBear,
                                         (["import os\n", "import sys\n"],
                                          ("import os\n", "import sys\n")),
                                         (["import sys\n", "import os\n"],
                                          ("import sys\n", "import os\n")))

PyImportSortBearConfigsTest = verify_local_bear(
    PyImportSortBear,
    (("from os import read\n", "from sys import *\n"),),
    (("from os import read\n", "from os import *\n"),),
    settings={"combine_star_imports": True})

PyImportSortBearIgnoredConfigsTest = verify_local_bear(
    PyImportSortBear,
    (("import xyz\n", "\n", "import abc\n"),
     ("from xyz import *\n", "\n", "import abc\n")),
    (("import xyz\n", "import abc\n"),
     ("import abc\n", "import xyz\n")),
    settings={"known_standard_library_imports": "xyz",
              "known_first_party_imports": "abc"})



from queue import Queue

from bears.general.LineCountBear import LineCountBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.results.Result import RESULT_SEVERITY, Result
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


class LineCountBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("name")
        self.uut = LineCountBear(self.section, Queue())

    def test_run(self):
        self.section.append(Setting("max_lines_per_file", 1))
        self.check_results(
            self.uut, ["line 1", "line 2", "line 3"],
            [Result.from_values("LineCountBear",
                                "This file had 3 lines, which is 2 lines more "
                                "than the maximum limit specified.",
                                severity=RESULT_SEVERITY.NORMAL,
                                file="default")],
            filename="default")
        self.check_validity(self.uut, ["1 line"])
        self.check_validity(self.uut, [])  # Empty file



import os

from bears.js.JSHintBear import JSHintBear
from tests.LocalBearTestHelper import verify_local_bear
from coalib.misc.ContextManagers import prepare_file

test_file1 = """
var name = (function() { return 'Anton' }());
""".splitlines(keepends=True)


test_file2 = """
function () {
}()
""".splitlines(keepends=True)


test_file3 = """
var a = (function() {
  return 0;
}());
""".splitlines(keepends=True)


jshintconfig = os.path.join(os.path.dirname(__file__),
                            "test_files",
                            "jshintconfig.json")


settings = {
    "maxstatements": "False",
    "maxparams": 10,
    "prohibit_unused": "False",
    "shadow": "False",
    "allow_last_semicolon": "True",
    "es_version": 3,
    "allow_latedef": "no_func"}


JSHintBearTest = verify_local_bear(JSHintBear,
                                   valid_files=(),
                                   invalid_files=(test_file1, test_file2,
                                                  test_file3))


JSHintBearConfigFileTest = verify_local_bear(
    JSHintBear,
    valid_files=(test_file1,),
    invalid_files=(test_file2,),
    settings={"jshint_config": jshintconfig})


JSHintBearCoafileTest = verify_local_bear(
    JSHintBear,
    invalid_files=(),
    valid_files=(test_file3, ),
    settings=settings)



import os
import unittest
from queue import Queue

from clang.cindex import Index
from coalib.results.Result import Result
from coalib.results.SourceRange import SourceRange
from coalib.settings.Section import Section

from bears.c_languages.ClangComplexityBear import (
    ClangComplexityBear)
from tests.BearTestHelper import generate_skip_decorator
from tests.LocalBearTestHelper import execute_bear


@generate_skip_decorator(ClangComplexityBear)
class ClangComplexityBearTest(unittest.TestCase):

    def setUp(self):
        self.filename = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                     "codeclone_detection",
                                                     "conditions_samples.c"))
        self.file = "fake"
        self.queue = Queue()
        self.section = Section("test section")
        self.bear = ClangComplexityBear(self.section, self.queue)

    def test_calculation(self):
        """
        Testing that number of decision points and exit points are calculated
        correctly.
        """
        expected = [
            ('used(int, int)', 3),
            ('returned(int, int)', 1),
            ('loopy(int, int)', 5),
            ('in_condition(int, int)', 1),
            ('assignation(int, int)', 2),
            ('arithmetics(int, int)', 1),
            ('levels(int, int, int)', 10),
            ('structing(struct test_struct, struct test_struct *)', 1),
            ('switching(int, int)', 2)]

        root = Index.create().parse(self.filename).cursor
        complexities_gen = self.bear.complexities(root, self.filename)
        results = [(cursor.displayname, complexity)
                   for cursor, complexity in complexities_gen]
        self.assertSequenceEqual(results, expected)

    def test_output(self):
        """
        Validating that the yielded results are correct.
        """
        affected_code = (SourceRange.from_values(
            self.filename,
            start_line=111,
            start_column=1,
            end_line=143,
            end_column=2),)
        expected_result = Result(
            self.bear,
            "The function 'levels(int, int, int)' should be simplified. Its "
            "cyclomatic complexity is 10 which exceeds maximal recommended "
            "value of 8.",
            affected_code=affected_code)
        with execute_bear(self.bear, self.filename, self.file, 8) as out:
            self.assertEqual(len(out), 1)
        out[0].additional_info = ""  # Let's not test this, static and huge
        self.assertEqual(out[0], expected_result)

    def test_empty_declared_function(self):
        """
        Should not take into account and display empty function declarations.
        """
        self.filename = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                     "test_files",
                                                     "empty_declarations.c"))
        expected = [('with_body(int *)', 1)]

        root = Index.create().parse(self.filename).cursor
        complexities_gen = self.bear.complexities(root, self.filename)
        results = [(cursor.displayname, complexity)
                   for cursor, complexity in complexities_gen]
        self.assertSequenceEqual(results, expected)

    def test_file_does_not_exist(self):
        """
        Tests that bear throws TranslationUnitLoadError when file does not
        exist.
        """
        from clang.cindex import TranslationUnitLoadError

        generator = self.bear.execute("not_existing", self.file)
        self.assertNotEqual(generator, None)
        with self.assertRaises(TranslationUnitLoadError):
            yield generator



from bears.c_languages.GNUIndentBear import GNUIndentBear
from tests.LocalBearTestHelper import verify_local_bear

test_file1 = """
int
main ()
{
    return 0;
}""".splitlines(keepends=True)

test_file2 = """
int
main ()
{
  return 0;
}""".splitlines(keepends=True)


test_file3 = """
int
main ()
{
\treturn 0;
}""".splitlines(keepends=True)


test_file4 = """
int main() {
  return 0;
}""".splitlines(keepends=True)


GNUIndentBearTest = verify_local_bear(
    GNUIndentBear,
    valid_files=(test_file1,),
    invalid_files=(test_file2, test_file3, test_file4),
    settings={"use_spaces": "true", "max_line_length": "80"})


GNUIndentBearWithTabTest = verify_local_bear(
    GNUIndentBear,
    valid_files=(test_file3,),
    invalid_files=(test_file1, test_file2, test_file4),
    settings={"use_spaces": "nope", "max_line_length": "80"})


GNUIndentBearGNUIndentWidthTest = verify_local_bear(
    GNUIndentBear,
    valid_files=(test_file2,),
    invalid_files=(test_file1, test_file3, test_file4),
    settings={"use_spaces": "true", "max_line_length": "80", "tab_width": "2"})



import json

from coalib.bearlib.abstractions.Linter import linter
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.results.Result import Result


@linter(executable='coffeelint')
class CoffeeLintBear:
    """
    Check CoffeeScript code for a clean and consistent style.

    For more information about coffeelint, visit <http://www.coffeelint.org/>.
    """

    LANGUAGES = "CoffeeScript"

    severity_map = {'warn': RESULT_SEVERITY.NORMAL,
                    'error': RESULT_SEVERITY.MAJOR}

    @staticmethod
    def create_arguments(filename, file, config_file):
        return '--reporter=raw', filename

    def process_output(self, output, filename, file):
        output = json.loads(output)

        assert len(output) == 1, "More than 1 file parsed, something went wrong"
        for item in tuple(output.values())[0]:
            yield Result.from_values(
                origin="{} ({})".format(self.name, item['rule']),
                message=item['message'],
                file=filename,
                line=item.get('lineNumber', None),
                end_line=item.get('lineNumberEnd', None),
                severity=self.severity_map[item['level']],
                additional_info=item.get('description',
                                         item.get('context', "")))



import os
from queue import Queue

from bears.java.CheckstyleBear import CheckstyleBear
from tests.BearTestHelper import generate_skip_decorator
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


@generate_skip_decorator(CheckstyleBear)
class CheckstyleBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = CheckstyleBear(self.section, Queue())
        test_files = os.path.join(os.path.dirname(__file__), "test_files")
        self.good_file = os.path.join(test_files, "CheckstyleGood.java")
        self.bad_file = os.path.join(test_files, "CheckstyleBad.java")
        self.empty_config = os.path.join(test_files,
                                         "checkstyle_empty_config.xml")

    def test_run(self):
        self.check_validity(self.uut, [], self.good_file)
        self.check_validity(self.uut, [], self.bad_file, valid=False)

    def test_known_configs(self):
        self.section["checkstyle_configs"] = "google"
        self.check_validity(self.uut, [], self.good_file)

    def test_with_custom_configfile(self):
        self.section["checkstyle_configs"] = self.empty_config
        self.check_validity(self.uut, [], self.good_file)
        self.check_validity(self.uut, [], self.bad_file)



from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class CSSAutoPrefixBear(Lint, LocalBear):
    executable = "postcss"
    arguments = "--use autoprefixer {filename}"
    prerequisite_command = ['postcss', '--use', 'autoprefixer']
    prerequisite_fail_msg = "Autoprefixer is not installed."
    diff_message = "Add vendor prefixes to CSS rules."
    gives_corrected = True
    LANGUAGES = "CSS"

    def run(self, filename, file):
        '''
        This bear adds vendor prefixes to CSS rules using ``autoprefixer``
        utility.
        '''
        return self.lint(filename, file)



from bears.java.JavaPMDBear import JavaPMDBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
package hello;
class Hello {
  public String myString;
  /* Default Hello constructor */
  public Hello(){
    myString = "Hello";
  }

  /* Print string length */
  public int test() {
    return myString.length();
  }
}
""".splitlines(keepends=True)

bad_file = """
// Hello.java
class Hello {
  int test() {
    String s = null;
    return s.length();
  }
}
""".splitlines(keepends=True)


JavaPMDBearTest = verify_local_bear(
    JavaPMDBear, valid_files=(good_file,), invalid_files=(bad_file,),
    tempfile_kwargs={"suffix": ".java"})



from bears.go.GoTypeBear import GoTypeBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
package main
import "fmt"
func main() {
    fmt.Println("Hello World!")
}
""".splitlines(keepends=True)


bad_file = """
package main
import (
    "fmt"
    "math/rand"
    "time"
)
type A struct{}

func (A) String() string { return "A" }
func (B) String() string { return "B" }

func Print(s fmt.Stringer) {
    fmt.Println("Hello " + s.String())
}
func main() {
    rand.Seed(time.Now().UnixNano())
    if rand.Intn(2) == 1 {
        Print(A{})
    } else {
        Print(B{})
    }
}
""".splitlines(keepends=True)


GoTypeBearTest = verify_local_bear(GoTypeBear,
                                   valid_files=(good_file,),
                                   invalid_files=(bad_file,))



import json

from coalib.bearlib.abstractions.Linter import linter


@linter(executable='remark',
        use_stdin=True,
        output_format='corrected',
        diff_message='The text does not comply to the set style.')
class MarkdownBear:
    """
    Check and correct Markdown style violations automatically.

    See <https://github.com/wooorm/remark-lint> for details about the tool
    below.
    """

    LANGUAGES = "Markdown"

    @staticmethod
    def create_arguments(filename, file, config_file,
                         markdown_bullets: str="-",
                         markdown_closed_headings: bool=False,
                         markdown_setext_headings: bool=False,
                         markdown_emphasis: str="*",
                         markdown_strong: str="*",
                         markdown_encode_entities: bool=False,
                         markdown_codefence: str="`",
                         markdown_fences: bool=True,
                         markdown_list_indent: str="1",
                         markdown_loose_tables: bool=False,
                         markdown_spaced_tables: bool=True,
                         markdown_list_increment: bool=True,
                         markdown_horizontal_rule: str='*',
                         markdown_horizontal_rule_spaces: bool=False,
                         markdown_horizontal_rule_repeat: int=3):
        """
        :param markdown_bullets:
            Character to use for bullets in lists. Can be "-", "*" or "+".
        :param markdown_closed_headings:
            Whether to close Atx headings or not. if true, extra # marks will
            be required after the heading. eg: `## Heading ##`.
        :param markdown_setext_headings:
            Whether to use setext headings. A setext heading uses underlines
            instead of # marks.
        :param markdown_emphasis:
            Character to wrap strong emphasis by. Can be "_" or "*".
        :param markdown_strong:
            Character to wrap slight emphasis by. Can be "_" or "*".
        :param markdown_encode_entities:
            Whether to encode symbols that are not ASCII into special HTML
            characters.
        :param markdown_codefence:
            Used to find which characters to use for code fences. Can be "`" or
            "~".
        :param markdown_fences:
            Use fences for code blocks.
        :param markdown_list_indent:
            Used to find spacing after bullet in lists. Can be "1", "tab" or
            "mixed".

            - "1" - 1 space after bullet.
            - "tab" - Use tab stops to begin a sentence after the bullet.
            - "mixed" - Use "1" when the list item is only 1 line, "tab" if it
              spans multiple.
        :param markdown_loose_tables:
            Whether to use pipes for the outermost borders in a table.
        :param markdown_spaced_tables:
            Whether to add space between pipes in a table.
        :param markdown_list_increment:
            Whether an ordered lists numbers should be incremented.
        :param markdown_horizontal_rule:
            The horizontal rule character. Can be '*', '_' or '-'.
        :param markdown_horizontal_rule_spaces:
            Whether spaces should be added between horizontal rule characters.
        :param markdown_horizontal_rule_repeat:
            The number of times the horizontal rule character will be repeated.
        """
        remark_configs = {
            "bullet": markdown_bullets,                         # - or *
            "closeAtx": markdown_closed_headings,               # Bool
            "setext": markdown_setext_headings,                 # Bool
            "emphasis": markdown_emphasis,                      # char (_ or *)
            "strong": markdown_strong,                          # char (_ or *)
            "entities": markdown_encode_entities,               # Bool
            "fence": markdown_codefence,                        # char (~ or `)
            "fences": markdown_fences,                          # Bool
            "listItemIndent": markdown_list_indent,             # int or "tab"
                                                                # or "mixed"
            "looseTable": markdown_loose_tables,                # Bool
            "spacedTable": markdown_spaced_tables,              # Bool
            "incrementListMarker": markdown_list_increment,     # Bool
            "rule": markdown_horizontal_rule,                   # - or * or _
            "ruleSpaces": markdown_horizontal_rule_spaces,      # Bool
            "ruleRepetition": markdown_horizontal_rule_repeat,  # int
        }

        config_json = json.dumps(remark_configs)
        # Remove { and } as remark adds them on its own
        settings = config_json[1:-1]
        return '--no-color', '--quiet', '--setting', settings
